/***
Demonstrates usage of the periodic stream-out function.

Streams out arbitrary values. These arbitrary stream-out values act on DAC0
to cyclically increase the voltage from 0 to 2.5 V.

Note: This example requires LJM 1.21 or higher.

Relevant Documentation:

LJM Library:
	LJM Library Installer:
		https://labjack.com/support/software/installers/ljm
	LJM Users Guide:
		https://labjack.com/support/software/api/ljm
	Opening and Closing:
		https://labjack.com/support/software/api/ljm/function-reference/opening-and-closing
	NamesToAddresses:
		https://labjack.com/support/software/api/ljm/function-reference/utility/ljmnamestoaddresses
	LJM Single Value Functions (such as eReadName, eReadAddress):
		https://labjack.com/support/software/api/ljm/function-reference/single-value-functions
	Stream Functions (such as eStreamStart, eStreamStop and PeriodicStreamOut):
		https://labjack.com/support/software/api/ljm/function-reference/stream-functions

T-Series and I/O:
	Modbus Map:
		https://labjack.com/support/software/api/modbus/modbus-map
	Stream Mode:
		https://labjack.com/support/datasheets/t-series/communication/stream-mode
	Stream-Out:
		https://labjack.com/support/datasheets/t-series/communication/stream-mode/stream-out
	Digital I/O:
		https://labjack.com/support/datasheets/t-series/digital-io
	DAC:
		https://labjack.com/support/datasheets/t-series/dac
***/
import com.sun.jna.ptr.DoubleByReference;
import com.sun.jna.ptr.IntByReference;

import java.io.Console;

import com.labjack.LJM;
import com.labjack.LJMException;

public class PeriodicStreamOut {

	public static void main(final String[] args) {
		try {
			IntByReference handleRef = new IntByReference(0);
			int handle = 0;

			//Open first found LabJack

			//Any device, Any connection, Any identifier
			LJM.openS("ANY", "ANY", "ANY", handleRef);

			//T8 device, Any connection, Any identifier
			//LJM.openS("T8", "ANY", "ANY", handleRef);

			//T7 device, Any connection, Any identifier
			//LJM.openS("T7", "ANY", "ANY", handleRef);

			//T4 device, Any connection, Any identifier
			//LJM.openS("T4", "ANY", "ANY", handleRef);

			//Any device, Any connection, Any identifier
			//LJM.open(LJM.Constants.dtANY, LJM.Constants.ctANY, "ANY",
			//		handleRef);

			handle = handleRef.getValue();

			LJMUtilities.printDeviceInfo(handle);

			//Scans per second
			double scanRate = 1000;
			int scansPerRead = (int)(scanRate / 2);

			//Desired duration to run the stream out
			int runTimeMS = 5000;
			int numAddresses = 1;
			String[] aScanListNames = new String[] { "STREAM_OUT0" };  //Scan list names to stream.
			int[] aScanList = new int[numAddresses];  //Scan list addresses to stream. eStreamStart uses Modbus addresses.
			int[] aTypes = new int[numAddresses];  //Dummy
			LJM.namesToAddresses(numAddresses, aScanListNames, aScanList, aTypes);
			int targetAddr = 1000;  // DAC0

			//With current T-series devices, 4 stream-outs can be ran concurrently.
			//Stream-out index should therefore be a value 0-3.
			int streamOutIndex = 0;
			int samplesToWrite = 512;

			//Make an arbitrary waveform that increases voltage linearly from 0-2.5V
			double[] values = new double[samplesToWrite];
			double increment = 1.0 / samplesToWrite;
			double sample = 0;
			for (int i = 0; i < samplesToWrite; i++) {
				sample = 2.5*increment*i;
				values[i] = sample;
			}

			try
			{
				System.out.println("\nInitializing stream out...");
				LJM.periodicStreamOut(
					handle,
					streamOutIndex,
					targetAddr,
					scanRate,
					samplesToWrite,
					values
				);

				System.out.println("\nBeginning stream out...");
				DoubleByReference scanRateRef = new DoubleByReference(scanRate);
				LJM.eStreamStart(handle, scansPerRead, numAddresses, aScanList, scanRateRef);
				Thread.sleep(runTimeMS);  //Delay for the desired runtime
			}
			catch (LJMException le) {
				le.printStackTrace();
			}
			catch (InterruptedException e) {
				e.printStackTrace();
			}

			System.out.println("Stopping Stream...");
			LJM.eStreamStop(handle);

			//Close handle
			LJM.close(handle);
		}
		catch (LJMException le) {
			le.printStackTrace();
			LJM.closeAll();
		}
	}
}
